@@ -3,6 +3,9 @@ adjacentreaches <- function(x, ...) UseMethod("adjacentreaches")
 
 adjacentreaches.rivernet <- function(x,crit.reach,crit.node=TRUE,thresh.length=0,...)
 {
+  
+  # function to extend a region; to be applied iteratively:
+  
   extend.region <- function(ind,crit.reach,crit.node,thresh.length,dist,
                             regions,attrib.reach)
   {
@@ -40,6 +43,8 @@ adjacentreaches.rivernet <- function(x,crit.reach,crit.node=TRUE,thresh.length=0
     return(regions)
   }
   
+  # initial checks:
+  
   if ( anyNA(crit.reach) )
   {
     cat("*** variable \"crit.reach\" contains NA ***\n")
@@ -73,6 +78,8 @@ adjacentreaches.rivernet <- function(x,crit.reach,crit.node=TRUE,thresh.length=0
     crit.node <- rep(crit.node,n.node)
   }
   
+  # main section to determine regions:
+
   regions <- rep(NA,n.reach)
   region <- 0
   while ( sum( is.na(regions) & crit.reach ) > 0 )
@@ -84,6 +91,47 @@ adjacentreaches.rivernet <- function(x,crit.reach,crit.node=TRUE,thresh.length=0
                              regions,rivernet$attrib.reach)
   }
   if ( sum(is.na(regions)) > 0 ) regions[is.na(regions)] <- 0
+  
+  # remove non-internal reaches that do not fulfill the criterion:
+  
+  n.regions <- max(regions)
+  if ( n.regions > 0 )
+  {
+    for ( reg in 1:n.regions )
+    {
+      r <- which(regions==reg & !crit.reach)   # reaches in region reg that do not fulfill the criterion
+      if ( length(r) > 0 )
+      {
+        # determine adjacent reaches:
+        r.adj <- list()
+        for ( j in 1:length(r) )
+        {
+          r.adj[[j]] <- list()
+          r.adj[[j]]$start <- which(rivernet$attrib.reach$node_start==rivernet$attrib.reach$node_start[r[j]] |
+                                    rivernet$attrib.reach$node_end  ==rivernet$attrib.reach$node_start[r[j]] )
+          r.adj[[j]]$end   <- which(rivernet$attrib.reach$node_start==rivernet$attrib.reach$node_end[r[j]] |
+                                    rivernet$attrib.reach$node_end  ==rivernet$attrib.reach$node_end[r[j]])
+        }
+        while ( TRUE )
+        {
+          found <- FALSE
+          for ( j in 1:length(r) )
+          {
+            if ( regions[r[j]] == reg )
+            {
+              if ( sum(regions[r.adj[[j]]$start] == reg) < 2 | sum(regions[r.adj[[j]]$end] == reg) < 2 )  # the region itself counts as adjacent too
+              {
+                regions[r[j]] <- 0
+                found <- TRUE
+              }
+            }
+          }
+          if ( !found ) break
+        }
+      }
+    }
+  }
+
   return(regions)
 }
 
